;*    AutoLISP Files for Inside AutoLISP IL DISK - (c) 1987, 1988, 1989
;* New Riders Publishing. All Rights Reserved. Version 10.00 for PC-DOS/MS-DOS
;*       Developed by Rustin Gesner, Patrick Haessly and Joseph Smith
;*
;*; DO NOT LSPSTRIP - MERGEF contains ;'s
;* FILELIB.LSP contains several file handling utility routines

;* CPATH returns current DOS or UNIX path: "d:\\path\\...\\" or "/path/.../".
;* WARNING: If DOS 3.3 APPEND is used results may be FALSE UNPREDICTABLY.
;* If SH (shell) fails with insufficient memory, increase its size in ACAD.PGP.
;*
(defun cpath ( / path fp slash)
                            ;creates 0 byte file, overwrite if existing...
  (close (open "$" "w"))    ;...allows READ-LINE to test SHell's success
  (command "SH"             ;redirect current dir to temp file $
    (cond 
      ( (getenv "COMSPEC") (setq slash "\\") "CD   >  $")          ;if DOS
      ( (getenv "USER")    (setq slash "/" ) "dirs >! $")          ;UNIX
      (T(prompt "\nUNKNOWN OPERATING SYSTEM for CPATH function. "))
  ) );command&cond
  (setq fp (open "$" "r"))
  (if (setq path (read-line fp))      ;nil only if SH failed
    (setq path (strcat path slash))   ;adds trailing slash to path
                                      ;sets path to "" if SH failed
    (progn (prompt "\nSHell FAILED!") (setq path ""))
  )
  (close fp)
  path                       ;returns path, which is "" only if SHell failed
);defun CPATH
;*

;* RPATH Resets the PATH to the "current AutoCAD directory" if changed by user.
;* For DOS only. Requires CPATH function.
;*
(defun rpath ( / path)
  (if (equal                               ;if ACAD current matches true current
        (cpath)
        (setq path (findfile "nul")                         ;gets "PATH/NUL"
              path (substr path 1 (- (strlen path) 3))      ;strips last 3 char
      ) )
      nil                                                               ;then nil
      (command "SH" (strcat "CD " (substr path 1 (- (strlen path) 1)))) ;else CD
  );if
);defun
;*

;* PSLASH PathSLASH converts "\\"s or "/"s in path strings to whichever is 
;* needed by operating system (UNIX or DOS), and forces trailing "\\" or "/".
(defun pslash (path / slash inc wpath char)
  (setq inc 1  wpath ""                                ;initialize variables
        slash (if (getenv "COMSPEC") "\\"  "/")        ;set for DOS or UNIX
  )
  (while (/= "" (setq char (substr path inc 1)))       ;test each char
    (setq wpath                                        ;append proper char back
      (strcat wpath (if (member char '("\\" "/")) slash char))
      inc (1+ inc)                                     ;increment counter
  ) );while&setq
  (if                                                  ;if last char isn't slash
    (and (/= wpath "") (/= (substr wpath (strlen wpath) 1) slash))
    (setq wpath (strcat wpath slash))                  ;make it a slash
  );if 
  wpath
);defun PSLASH
;*

;* FFNAME formats a filename as "filename.ext" given input FNAME with or  
;* without extension, and EXT as "EXT". 
(defun ffname (fname ext / inc lngth)
  (setq inc -1  lngth (strlen fname))     ;initialize, lngth is filename length
  (while 
    (not                                  ;loops until OR is non-NIL
      (or                                 ;eval 2nd AND only if 1st is NIL
        (and                              ;setq FNAME only if "." = non-NIL
          (= "." (substr fname (- lngth (setq inc (1+ inc))) 1))   ;find "."
          (setq                           ;strip last char and append EXT
            fname (strcat (substr fname 1 (- lngth inc)) ext)
          ) 
        ) 
        (and                              ;setq FNAME only if... 
          (or (= inc 3) (= inc lngth))    ;...if "." not found in last 3 char 
          (setq fname (strcat fname "." ext)) ;then append EXT to whole fname
  ) ) ) )
  fname
);defun
;*

;* VFFILE verifies a file's existence. It returns fspec or NIL.
;* WARNING: DOS 3.3 APPEND may cause FALSE UNPREDICTABLE results.
;* May be called w/ fname specific path "\\path\\filename.ext", or just 
;* "filename.ext", which searches ACAD library path or prompts for path.
;* Uses PSLASH to return "/"s or "\\"s for UNIX or DOS.
(defun vffile (fspec / fname fp path char prmpt)
  (setq inc -1  lngth (strlen fspec)  fname "") ;initialize
  (while (and (/= lngth (1+ inc)) (not path))   ;parse FSPEC into PATH and FNAME
       (if (member                              ;if char is a slash
             (setq char                         ;get current char, back to front
                   (substr fspec (- lngth (setq inc (1+ inc))) 1)
             )
             '("/" "\\")
           );member
           (setq path (substr fspec 1 (- lngth inc)))   ;then set path to remainder
           (setq fname (strcat char fname))             ;else 
  )    )
  (if path nil (setq path ""))                          ;if no path, set "" null
  (while                                  ;while file can't be opened, reprompt
    (and 
      (/= "Q" path)                                     ;until Quit
      (not (setq fspec (findfile (strcat path fname)))) ;or found
    )
    (setq prmpt                                         ;build prompt
      (strcat "\n\nFile " fname " not found in " 
        (if (= "" path) "current directory," path)      ;current dir if path=""
        "\nEnter path to search, or Q to quit: "
      );strcat
      path (strcase (getstring prmpt))
  ) );while & setq
  (if (= "Q" path)                                      ;returns nil or fspec
      (progn (prompt "File not found. ") nil)           ;nil 
      (substr (pslash fspec) 1 (strlen fspec))          ;fspec
  )
);defun vffile
;*

;* VFPATH is for compatibility with original CA DISK programs which call it 
;* with a filename argument. It calls a substitute function, VPATH.
;* The FN argument is not used in VPATH.
(defun vfpath (fn) (vpath))
;*

;* VPATH gets and verifies a PATH. On non-DOS systems, the filename "$."  
;* will be left behind in the successful directory, as a 0 byte file.
;* Adding (command "FILES" "3" (strcat path "$") "" "") will delete it.
;* It calls CPATH to present a default path.
(defun vpath ( / char path fp def)
  (setq def (cpath))                                     ;default current path
  (while (not path)                                      ;loops until valid path
    (setq path                                           ;get path name
          (getstring (strcat "\nEnter path name <" def ">: "))
    )
    (if (/= "" path)                                     ;user wants current path
      (if (getenv "COMSPEC")                             ;then, if DOS
        (or (findfile (strcat path "nul"))
            (setq path (prompt "\nInvalid path, try again."))   ;path not found
        )
        (progn                                           ;not DOS
          (setq path (pslash path))                      ;check for valid path syntax
          (if (setq fp (open (strcat path "$") "a"))     ;test user's path entry
            (close fp)                                   ;path found, close temp file
            (setq path (prompt "\nInvalid path, try again.")) ;path could not be accessed
          );if
        );progn
      );if-getenv
      (setq path def)                                    ;else set to CPATH
    );if/=
  );while
  path                                                   ;return the path
);defun VPATH
;*

;* BACKUP copies file fspec to its path\name.BAK, using 
;* the FFNAME function to format the filename.
(defun backup (fspec)
  (command "FILES" "5" fspec   ;copy filespec.EXT to 
    (ffname fspec "BAK")       ;filespec w/ EXT stripped & repl w/ BAK
    "" "" nil                  ;nil cancels error if file didn't exist.
  );command 
);defun
;*

;* MERGEF appends file2 to file1, strips file1's ^Z if present, deletes file2
;* If SH (shell) fails w/ insufficient memory, increase size in ACAD.PGP
;* It uses the VFFILE function to verify the filenames' existence. Returns T.
(defun mergef (file1 file2)     
  (setq file1 (vffile file1)  file2 (vffile file2))      ;verifies files
  (cond 
    ( (getenv "COMSPEC")                               ;if DOS
      (command "SH" (strcat "COPY " file1 " + "  file2  " "  file1) nil) ;copy
      (command "SH" (strcat "DEL " file2) nil)         ;delete 2nd file
    ) 
    ( (getenv "USER")                                  ;if UNIX
      (command "SH" (strcat "cat " file1 file2 " >! $" ;use cat command
                      " ; mv $ " file1 " ; rm " file2  ;delete 2nd file
      ) nil         );command&strcat                   ;nil cancels if SH fails 
    )
    (T(prompt "\nUNKNOWN OPERATING SYSTEM for MERGEF function. "))
  );cond
  T                                     ;returns T
);defun
;*

(princ)
;* end of FILELIB.LSP
;*
